package com.kakao.sdk.auth

import com.kakao.sdk.auth.model.AccessTokenResponse
import com.kakao.sdk.common.ApiFactory
import com.kakao.sdk.common.util.Utility
import com.kakao.sdk.network.withClient
import io.reactivex.observers.TestObserver
import okhttp3.OkHttpClient
import okhttp3.mockwebserver.MockResponse
import okhttp3.mockwebserver.MockWebServer
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.params.ParameterizedTest
import org.junit.jupiter.params.provider.CsvFileSource

/**
 * @author kevin.kang. Created on 2018. 4. 25..
 */
class RxAuthApiRequestTest {
    private lateinit var api: RxAuthApi
    private lateinit var server: MockWebServer

    @BeforeEach
    fun setup() {
        server = MockWebServer()
        server.start()
        api = ApiFactory.withClient(server.url("/").toString(), OkHttpClient.Builder())
            .create(RxAuthApi::class.java)
        val response = MockResponse().setResponseCode(200)
        server.enqueue(response)
    }

    @CsvFileSource(resources = ["/csv/token_requests.csv"], numLinesToSkip = 1)
    @ParameterizedTest
    fun simple(
        clientId: String,
        redirectUri: String,
        approvalType: String,
        androidKeyHash: String,
        code: String?,
        refreshToken: String?,
        grantType: String
    ) {
        api.issueAccessToken(
            clientId, redirectUri, approvalType, androidKeyHash,
            authCode = code, refreshToken = refreshToken, grantType = grantType
        )
            .subscribe(TestObserver<AccessTokenResponse>())

        val request = server.takeRequest()
        val params = Utility.parseQuery(request.body.readUtf8())

        assertEquals("POST", request.method)
        print(params)
        assertEquals(clientId, params[Constants.CLIENT_ID])
        assertEquals(redirectUri, params[Constants.REDIRECT_URI])
        assertEquals(approvalType, params[Constants.APPROVAL_TYPE])
        assertEquals(androidKeyHash, params[Constants.ANDROID_KEY_HASH])
        assertEquals(code, params[Constants.CODE])
        assertEquals(grantType, params[Constants.GRANT_TYPE])
    }

    @AfterEach
    fun cleanup() {
        server.shutdown()
    }
}