import org.jetbrains.dokka.gradle.DokkaTask
import java.net.URL

// Top-level build file where you can add configuration options common to all sub-projects/modules.

buildscript {
    repositories {
        google()
        jcenter()
    }
    dependencies {
        classpath("com.android.tools.build:gradle:${Versions.gradle}")
        classpath("org.jetbrains.kotlin:kotlin-gradle-plugin:${Versions.kotlin}")
        // NOTE: Do not place your application dependencies here; they belong
        // in the individual module build.gradle.kts files
    }
}

allprojects {
    repositories {
        google()
        jcenter()
    }
}

plugins {
    id("org.jetbrains.dokka") version "0.10.0"
    `maven-publish`
}


val libraries = rootProject.subprojects.filter { project ->
    project.name !in arrayOf(
        "sample",
        "sample-common",
        "java-sample"
    )
}

val clean by tasks.registering(Delete::class) {
    delete(rootProject.buildDir)
}


val publishSdk by tasks.registering(Task::class) {
    dependsOn(zipProject)
    dependsOn(zipDokka)
}

val zipProject by tasks.registering(Zip::class) {
    val fullSourcePath = "${rootProject.buildDir}/full_source"
    copy {
        from(rootProject.rootDir)
        into(fullSourcePath)
        exclude(
            relativePath(rootProject.buildDir),
            "**/.gradle",
            ".idea",
            ".project.info",
            "**/jacoco.exec",
            "**/*.iml",
            "buildSrc/build"
        )
        exclude(
            rootProject.subprojects.map { relativePath(it.buildDir) }
        )
    }
    from(fullSourcePath)
    destinationDirectory.set(rootProject.buildDir)
    archiveBaseName.set(Publish.projectName)
    archiveVersion.set(SdkVersions.version)
}

val dokka by tasks.getting(DokkaTask::class) {
    outputDirectory = "${rootProject.buildDir}/dokka"
    subProjects = libraries.map { it.name }
    configuration {
        moduleName = Publish.projectName
        includes = listOf("packages.md")
        skipEmptyPackages = true
        Dokka.externalDocs.forEach {
            externalDocumentationLink {
                // Root URL of the generated documentation to link with. The trailing slash is required!
                url = URL(it)
                // If package-list file located in non-standard location
                packageListUrl = URL("${it}package-list")
            }
        }
        Dokka.externalPackages.forEach {
            perPackageOption {
                prefix = it
                suppress = true
            }
        }
    }
}

val zipDokka by tasks.registering(Zip::class) {
    dependsOn(dokka)
    from("${rootProject.buildDir}/dokka")
    destinationDirectory.set(rootProject.buildDir)
    archiveBaseName.set("${Publish.projectName}-doc")
    archiveVersion.set(SdkVersions.version)
    delete("${rootProject.buildDir}/dokka")
}

val nexusSnapshotRepositoryUrl: String by project
val nexusUsername: String by project
val nexusPassword: String by project

publishing {
    repositories {
        maven {
            url = uri(nexusSnapshotRepositoryUrl)
            credentials {
                username = nexusUsername
                password = nexusPassword

            }
        }
    }
    publications {
        register("dokka", MavenPublication::class) {
            groupId = Publish.groupId
            artifactId = Publish.dokkaArtifactId
            version = SdkVersions.version
            artifact(zipDokka.get())
        }
        register("project", MavenPublication::class) {
            groupId = Publish.groupId
            artifactId = Publish.projectArtifactId
            version = SdkVersions.version
            artifact(zipProject.get())

        }
    }
}