package com.kakao.sdk.auth

import android.app.Activity
import android.content.Intent
import android.content.ServiceConnection
import android.net.Uri
import android.os.Bundle
import android.os.ResultReceiver
import com.kakao.sdk.common.util.KakaoCustomTabsClient

/**
 * @suppress
 * @author kevin.kang. Created on 2018. 3. 24..
 */
class AuthCodeCustomTabsActivity : Activity() {
    private lateinit var resultReceiver: ResultReceiver
    private lateinit var fullUri: Uri
    private var customTabsConnection: ServiceConnection? = null
    private var customTabsOpened = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val extras = intent.extras ?: throw IllegalArgumentException()
        resultReceiver =
            extras.getParcelable<ResultReceiver>(Constants.KEY_RESULT_RECEIVER) as ResultReceiver
        fullUri = extras.getParcelable(Constants.KEY_FULL_URI) ?: throw IllegalArgumentException()
    }

    override fun onResume() {
        super.onResume()
        if (!customTabsOpened) {
            openChromeCustomTab(fullUri)
            customTabsOpened = true
        } else {
            finish()
        }
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        val uri = intent?.dataString
        val bundle = Bundle()
        if (uri == null) {
            resultReceiver.send(RESULT_CANCELED, bundle)
            finish()
            return
        }
        bundle.putParcelable(Constants.KEY_URL, intent.data)
        resultReceiver.send(RESULT_OK, bundle)
        finish()
    }

    fun openChromeCustomTab(uri: Uri) {
        customTabsConnection = KakaoCustomTabsClient.openWithDefault(this, uri)
    }

    override fun onDestroy() {
        super.onDestroy()
        customTabsConnection?.let { unbindService(it) }
    }
}