import org.jetbrains.dokka.gradle.DokkaTask
import java.net.URL

// Top-level build file where you can add configuration options common to all sub-projects/modules.

buildscript {
    repositories {
        google()
        jcenter()
    }
    dependencies {
        classpath("com.android.tools.build:gradle:${Versions.gradle}")
        classpath("org.jetbrains.kotlin:kotlin-gradle-plugin:${Versions.kotlin}")
        // NOTE: Do not place your application dependencies here; they belong
        // in the individual module build.gradle.kts files
    }
}

allprojects {
    repositories {
        google()
        jcenter()
    }
}

plugins {
    id("org.jetbrains.dokka") version "0.10.0"
}


val libraries = rootProject.subprojects.filter { project ->
    project.name !in arrayOf(
        "sample",
        "sample-common",
        "java-sample"
    )
}

tasks {
    val clean by registering(Delete::class) {
        delete(rootProject.buildDir)
    }

    val zipProject by registering(Zip::class) {
        val fullSourcePath = "${rootProject.buildDir}/full_source"
        copy {
            from(".")
            into(fullSourcePath)
            exclude(
                relativePath(rootProject.buildDir),
                "**/.gradle",
                ".idea",
                ".project.info",
                "**/jacoco.exec",
                "**/*.iml",
                "buildSrc/build"
            )
            exclude(
                rootProject.subprojects.map { relativePath(it.buildDir) }
            )
        }
        from(fullSourcePath)
        destinationDirectory.set(rootProject.buildDir)
        archiveBaseName.set(Publish.projectName)
        archiveVersion.set(SdkVersions.version)
//        delete(fullSourcePath)
    }

    val dokka by getting(DokkaTask::class) {
        outputDirectory = "${rootProject.buildDir}/dokka"
        subProjects = libraries.map { it.name }
        configuration {
            moduleName = Publish.projectName
            includes = listOf("packages.md")
            skipEmptyPackages = true
            Dokka.externalDocs.forEach {
                externalDocumentationLink {
                    // Root URL of the generated documentation to link with. The trailing slash is required!
                    url = URL(it)
                    // If package-list file located in non-standard location
                    packageListUrl = URL("${it}package-list")
                }
            }
            Dokka.externalPackages.forEach {
                perPackageOption {
                    prefix = it
                    suppress = true
                }
            }
        }
    }

    val zipDokka by registering(Zip::class) {
        dependsOn(dokka)
        from("${rootProject.buildDir}/dokka")
        destinationDirectory.set(rootProject.buildDir)
        archiveBaseName.set("${Publish.projectName}-doc")
        archiveVersion.set(SdkVersions.version)
        delete("${rootProject.buildDir}/dokka")
    }

    val buildSdk by registering(Task::class) {
        dependsOn(zipProject)
        dependsOn(zipDokka)

    }
}
