package com.kakao.sdk.talk.model

import android.os.Parcelable
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.common.json.UnknownValue
import com.kakao.sdk.talk.Constants
import kotlinx.android.parcel.Parcelize
import java.util.*

/**
 * /v1/api/talk/plusfriends 응답
 *
 * @property userId 요청자의 사용자 ID
 * @property plusFriends 각 플러스친구와 요청 사용자의 관계 정보
 *
 * @since 2.0.0
 *
 * @author kevin.kang. Created on 05/04/2019..
 */
@Parcelize
data class PlusFriendsRelations(val userId: Long, val plusFriends: List<PlusFriendRelation>) :
    Parcelable

/**
 * 유저와 플러스 친구의 관계 정보
 *
 * @property uuid plus friend uuid
 * @property encodedId encoded plus friend id (ex. https://pf.kakao.com/${plusFriendId})
 * @property relation 플친과의 관계. 추가된 상태 / 관계 없음.
 * @property updatedAt relation 변경 시각 (현재는 ADDED 상태의 친구 추가시각만 의미)
 * RFC3339 internet date/time format
 * (yyyy-mm-dd'T'HH:mm:ss'Z', yyyy-mm-dd'T'HH:mm:ss'+'HH:mm, yyyy-mm-dd'T'HH:mm:ss'-'HH:mm 가능)
 *
 * @author kevin.kang. Created on 05/04/2019..
 */
@Parcelize
data class PlusFriendRelation(
    @SerializedName(Constants.PLUS_FRIEND_UUID) val uuid: String,
    @SerializedName(Constants.PLUS_FRIEND_PUBLIC_ID) val encodedId: String,
    val relation: Relation,
    val updatedAt: Date
) : Parcelable {

    enum class Relation {
        /**
         * 추가된 상태
         */
        @SerializedName("ADDED")
        ADDED,
        /**
         * 관계없음
         */
        @SerializedName("NONE")
        NONE,
        @UnknownValue
        UNKNOWN;
    }
}

