package com.kakao.sdk.link

import android.net.Uri
import com.google.gson.JsonObject
import com.kakao.sdk.common.model.ApplicationInfo
import com.kakao.sdk.common.Constants as CommonConstants
import com.kakao.sdk.common.model.ContextInfo
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.template.DefaultTemplate

/**
 * @author kevin.kang. Created on 10/04/2019..
 */
class WebSharerClient(
    private val contextInfo: ContextInfo = KakaoSdk.applicationContextInfo,
    private val applicationInfo: ApplicationInfo = KakaoSdk.applicationContextInfo
) {

    fun customTemplateUri(
        templateId: Long,
        templateArgs: Map<String, String>? = null,
        serverCallbackArgs: Map<String, String>? = null
    ): Uri {
        val validationParams = JsonObject().apply {
            addProperty(Constants.TEMPLATE_ID, templateId)
            templateArgs?.let {
                addProperty(
                    Constants.TEMPLATE_ARGS,
                    KakaoJson.toJson(templateArgs)
                )
            }
            addProperty(Constants.LINK_VER, Constants.LINKVER_40)
        }
        val builder = baseUriBuilder(serverCallbackArgs)
            .appendQueryParameter(Constants.VALIDATION_ACTION, Constants.VALIDATION_CUSTOM)
            .appendQueryParameter(Constants.VALIDATION_PARAMS, validationParams.toString())
        return builder.build()
    }

    fun scrapTemplateUri(
        requestUrl: String,
        templateId: Long? = null,
        templateArgs: Map<String, String>? = null,
        serverCallbackArgs: Map<String, String>? = null
    ): Uri {
        val validationParams = JsonObject().apply {
            addProperty(Constants.REQUEST_URL, requestUrl)
            templateId?.let { addProperty(Constants.TEMPLATE_ID, it) }
            templateArgs?.let {
                addProperty(
                    Constants.TEMPLATE_ARGS,
                    KakaoJson.toJson(it)
                )
            }
            addProperty(Constants.LINK_VER, Constants.LINKVER_40)
        }
        val builder = baseUriBuilder(serverCallbackArgs)
            .appendQueryParameter(Constants.VALIDATION_ACTION, Constants.VALIDATION_SCRAP)
            .appendQueryParameter(Constants.VALIDATION_PARAMS, validationParams.toString())
        return builder.build()
    }

    fun defaultTemplateUri(
        templateParams: DefaultTemplate,
        serverCallbackArgs: Map<String, String>? = null
    ): Uri {
        val validationParams = JsonObject().apply {
            add(Constants.TEMPLATE_OBJECT, KakaoJson.base.toJsonTree(templateParams))
            addProperty(Constants.LINK_VER, Constants.LINKVER_40)
        }
        val builder = baseUriBuilder(serverCallbackArgs)
            .appendQueryParameter(Constants.VALIDATION_ACTION, Constants.VALIDATION_DEFAULT)
            .appendQueryParameter(Constants.VALIDATION_PARAMS, validationParams.toString())
        return builder.build()
    }

    private fun baseUriBuilder(serverCallbackArgs: Map<String, String>?): Uri.Builder {
        val builder = Uri.Builder().scheme(CommonConstants.SCHEME)
            .authority(KakaoSdk.serverHosts.sharer)
            .path(Constants.SHARER_PATH)
            .appendQueryParameter(Constants.SHARER_APP_KEY, applicationInfo.clientId)
            .appendQueryParameter(Constants.SHARER_KA, contextInfo.kaHeader)
        serverCallbackArgs?.let {
            builder.appendQueryParameter(
                Constants.LCBA,
                KakaoJson.toJson(serverCallbackArgs)
            )
        }
        return builder
    }

    companion object {
        val instance by lazy { WebSharerClient() }
    }
}